%% AP onset speed
% extract AP parameters during the transfer function protocol

clearvars
close all

% Getting all the directories of recorded cells that contain the transfer
% function protocol and deleting unnecessary folders
directories = dir('/Users/stefanobordabossana/Documents/Experiments/analysed cells with tf/*');
directories(1:3) = [];
directories(end) = [];

for i = 1:length(directories)% For loop over all the recorded cells
    
    cd([directories(i).folder '/' directories(i).name]);
    
    files = listH5Files([pwd '/tf/']); % Loading the names recordings
    
    kernels = dir([pwd '/tf/*_kernel.dat']);
    Ke = load(kernels(1).name); % Loading the kernels to perform electrode compensation
    
    % Extracting from the raw data the time, membrane voltage, injected
    % current
    [t,Vr,Iinj,stim] = loadH5Traces(files); 
    
    % Performing electrode compensation
    [Vc] = myAEC(Ke, Iinj, Vr, files, t, stim, 1);
    Iinj_step = Iinj;
    
    trace = Vc(1,:);
    
    [entitites, info] = loadH5Trace(files{1});
    
    % Getting the start-end times of the stimulation
    Iinj_step = Iinj(1,:);
    Iinj_step(Iinj_step ~= 0) = 1;
    stimulation_StartPoints = strfind(Iinj_step, [0 1]);
    stimulation_EndPoints = strfind(Iinj_step, [1 0]);
    
    % Finding all spikes
    [~, AP_locs] = findpeaks(Vc(1,stimulation_StartPoints(3)+1:stimulation_EndPoints(3)), 'MinPeakHeight', -20, 'MinPeakWidth', 20);
    
    % Extracting the noise part of the injected current
    Inoise(1,:) = Iinj(1 ,stimulation_StartPoints(3)+1:stimulation_EndPoints(3)) ...
        - mean(Iinj(1,stimulation_StartPoints(3)+1:stimulation_EndPoints(3)));
    
    % Find spikes during noise injection
    SpikeTrain = AP_locs((AP_locs > 10*info.srate) & (AP_locs < 20*info.srate*2));
    
    SpikeTrain = SpikeTrain + stimulation_StartPoints(3)+1;
    
    % Compute average spike shape
    startSpike = 30*30000/1000;
    endSpike = 65*30000/1000;
    timevec = 0 : t(2) : (startSpike + endSpike) * t(2);
    
    averagespike = mean(ActionPotentials(1:end, :));
    ap = averagespike;
    
    % Find and measure ADP, if present
    for k = 1:size(ap,1)
        
        % Extracting the part of the AP from its maximum to the end of the
        % AP
        adptrace = ap(k, find(ap(k,:) == max(ap(k,:))):end);
        adp1 = diff(adptrace)./t(2);
        adp1_sma = smooth(adp1);
        
        % Finding the ADP
        ind = find(adp1_sma > 0, 1, 'first') - 1;
        
        [AHPpeak, AHPind] = min(adptrace);
        
        % Check for legitimacy of ADP
        if AHPind > ind
            
            [adpPeak(k), adpInd(k)] = max(adptrace(ind:AHPind));
            adpInd(k) = adpInd(k) + ind;
            
            if adpInd(k) < ind+10
                
                adpAmp(k,:) = 0;
                
            else
                
                adpAmp(k,:) = adpPeak(1,k) - AHPpeak;
                
            end
            
        else
            
            adpAmp(k,:) = 0;
            
        end
        
    end
    
    % Measuring the other AP parameters
    for j = 1:size(ap, 1)
        
        spike = ap(j,:);
        
        spike1 = diff(spike)./(t(2)*1000); % AP first derivative
        
        % the threshold was calculated as the potential where the change in
        % voltage over time is 20 mV/ms
        thresInd = find(spike1 > 20, 1);
        
        threshold = ap(j, thresInd);
        
        spike2 = diff(spike1) ./ diff(spike(1:end-1)); % AP second derivative
        
        slope_APonset = spike1(thresInd+1);
        speed_APonset = spike2(thresInd-1);
        
        [peak, peakInd] = max(ap(j,:)); % AP peak
        
        % the AP half-width (in milliseconds) is calculated as the difference in
        % time between the ascending and descending phases of a spike at the
        % voltage midpoint between spike peak and spike threshold.
        voltageMidpoint = (peak + threshold)/2;
        midpoint1 = find(ap(j,:) > voltageMidpoint, 1);
        midpoint2 = find(ap(j,midpoint1:end) < voltageMidpoint, 1, 'first');
        midpoint2 = midpoint2 + midpoint1;
        
        halfWidth = (midpoint2 - midpoint1)*(t(2)*1000);
        
        % the AP amplitude (in millivolts) is calculated as the difference in potential
        % from the spike peak to the spike threshold.
        amplitude = peak - threshold;
        
        % AP duration is defined as the average time for the first AP from onset
        % to the same voltage during offset.
        offset = find(ap(j,:) > threshold, 1, 'last');
        duration = (offset - thresInd)*(t(2)*1000);
        
        % AP rise time is the duration from onset to the peak.
        % AP fall time is the duration from peak to the offset.
        riseTime = (peakInd - thresInd)*(t(2)*1000);
        fallTime = (offset - peakInd)*(t(2)*1000);
        
        % rise rate is the AP amplitude divided by the rise time
        % fall rate is the AP amplitude divided by the fall time
        riseRate = amplitude/riseTime;
        fallRate = amplitude/fallTime;
        
        [fAHP_peak, fAHP_ind] = min(ap(j, peakInd:end));
        fAHP_ind = fAHP_ind + length(ap(j, 1:peakInd-1));
        fAHP_amp = threshold - fAHP_peak;
        
        % The maximum spike slope (in millivolts/millisecond) was calculated as
        % the maximum change in potential over time for the first observed spike.
        MaxSpikeSlope = max(spike1(thresInd:peakInd));
        
        % Plotting the data
        figure(3)
        
        hold on
        set(gcf, 'color', 'white');
        box off
        
        plot(timevec, ap(j,:), 'k');
        plot(thresInd*t(2), threshold, 'r.', 'MarkerSize', 15);
        plot(peakInd*t(2), peak, 'b.', 'MarkerSize', 15);
        plot(fAHP_ind*t(2), fAHP_peak, 'g.', 'MarkerSize', 15);
        
        if adpAmp(j) ~= 0
            plot((adpInd(1,j) + find(ap(j,:) == max(ap(j,:))))*t(2), adpPeak(1,j), 'm.', 'MarkerSize', 15);
        end
        
        ylabel('membrane potential, mV');
        ax =  gca();
        ax.XAxis.Visible = 'off';
        
        text(0.04, 0, sprintf(' peak = %g mV\n threshold = %g mV\n amplitude = %g mV\n duration = %g ms\n half-width = %g ms\n fAHP = %g mV\n ADP = %g mV\n slope at onset = %g mV/ms\n speed of onset = %g mV/ms', ...
            peak, threshold, amplitude, duration, halfWidth, fAHP_amp, adpAmp(j), slope_APonset, speed_APonset), 'FontSize', 12);
        
        % Saving the data
        AP_noise.spike = ap(1,:);
        AP_noise.threshold = threshold;
        AP_noise.peak = peak;
        AP_noise.amplitude = amplitude;
        AP_noise.duration = duration;
        AP_noise.halfWidth = halfWidth;
        AP_noise.riseTime = riseTime;
        AP_noise.fallTime = fallTime;
        AP_noise.riseRate = riseRate;
        AP_noise.fallRate = fallRate;
        AP_noise.MaxSpikeSlope = MaxSpikeSlope;
        AP_noise.fAHP_amplitude = fAHP_amp;
        AP_noise.fAHP_peak = fAHP_peak;
        AP_noise.slopeatAPonset = slope_APonset;
        AP_noise.speedatAPonset = speed_APonset;
        
    end
    
    pathtosave = [pwd '/tf'];
    save([pathtosave '/AP_noise.mat'], 'AP_noise');
    saveas(figure(3), fullfile ([pathtosave '/'], 'ap_noise'), 'fig');
    
    close all
    clearvars -except directories
    
end